#	$Id: offsetdraw.py,v 1.1 2006-07-16 10:02:13 gerard Exp $
# -*- coding: latin-1 -*-
## ********************************************************************************
## 
## Copyright (C) 2005 Grard Deneux
## gerard.deneux@free.fr

## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.

## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## Lesser General Public License for more details.
## http://www.opensource.org/licenses/lgpl-license.php

## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
## ******************************************************************************
try:
    import uno, unohelper
    from com.sun.star.task import XJob
    from com.sun.star.drawing.CircleKind import FULL, SECTION, CUT, ARC
    import math, traceback
except ImportError:
    print "probleme d'import"
#======================================================
# a class to draw an offset copy of an entity
# une classe pour tracer une copie dcale d'une entit
#======================================================
class OffsetDraw(unohelper.Base, XJob):
    """Trace une copie dcale  d'une entite graphique
    Draws an offset copy of a graphical entity"""
    def __init__(self, ctx):
        self.ctx = ctx
        desktop = self.ctx.ServiceManager.createInstanceWithContext( "com.sun.star.frame.Desktop",self.ctx)
        # access the current draw document
        self.aEntitySpot = self.ctx.ServiceManager.createInstanceWithContext("org.openoffice.comp.pyuno.EntitieSpot", self.ctx)
    # =========================================
    # wrapper for the compute functions
    # =========================================
    def compute(self, sMethod, *aArgs):
        aCompute = self.ctx.ServiceManager.createInstanceWithContext("org.openoffice.comp.pyuno.Compute", self.ctx)
        aComputeNamedValues = []
        n = len(aArgs) + 2
        for i in range(n):
            aComputeNamedValues.append(uno.createUnoStruct("com.sun.star.beans.NamedValue"))
            if i == 0:
                aComputeNamedValues[i].Name = "Function"
                aComputeNamedValues[i].Value = sMethod
            elif i < n-1:
                aComputeNamedValues[i].Name = "Param%d" % i
                aComputeNamedValues[i].Value = aArgs[i-1]
            else:
                aComputeNamedValues[i].Name = "Result"
        aComputeNamedValues = list(aCompute.execute(tuple(aComputeNamedValues),))
        return aComputeNamedValues[n-1].Value
    # ==============================
    def execute(self, aNamedValues):
        self.aEnv = dict(list(aNamedValues[0].Value))
        aNamedValues[1].Value = "spot"
        self.aEnv = dict(list(self.aEntitySpot.execute(aNamedValues)[0].Value))
        # -----------------------------------
        if not self.aEnv['SPOTLIST'] is None: # entity has been spotted ?
            # ..................................................................................................
            if self.aEnv['SPOTTEDID'] == self.aEnv['SPOTTEDSHAPE'][0]+(self.aEnv['SPOTTEDSHAPE'][-1],):
                self.aEnv['STEP'] = 0
                aNamedValues[0].Value = tuple(self.aEnv.items())
                return aNamedValues
            self.removeTempShape()
            self.aEnv['SPOTTEDID'] = self.aEnv['SPOTTEDSHAPE'][0][:] + (self.aEnv['SPOTTEDSHAPE'][-1],)
            aGroupShape = self.aEnv['CONTROLLER'].getCurrentPage()
            for i in self.aEnv['SPOTTEDID'][:-2]:
                aGroupShape = aGroupShape.getByIndex(i)
            i = self.aEnv['SPOTTEDSHAPE'][0][-1]
            aShape = aGroupShape.getByIndex(i)
            aType = aShape.getShapeType().split('.')[-1]
            if self.aEnv['MOUSERELPOS'][1] != 0:
                nSign = self.aEnv['MOUSERELPOS'][1] / abs(self.aEnv['MOUSERELPOS'][1])
            else:
                nSign = 1
            aCoords = []
            self.removeTempShape()
            # .........................
            if aType in ('LineShape',):
                self.aEnv['UNOTYPE'] = "LineShape"
                self.aEnv['ENTITYPE'] = "LINE"
                self.aEnv['STEP'] = 1
                self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("VERTEX",)
                aCoords.append([])
                aCoords[0].append(self.compute("CoordFromAbsOrd",\
                                                aShape.PolyPolygon[0][0],\
                                                aShape.PolyPolygon[0][1],\
                                                0,\
                                                nSign * self.aEnv['OFFSET']))
                aCoords[0].append(self.compute("CoordFromAbsOrd",\
                                                aShape.PolyPolygon[0][0],\
                                                aShape.PolyPolygon[0][1],\
                                                self.aEnv['MOUSERELPOS'][2],\
                                                nSign * self.aEnv['OFFSET']))
            # .............................
            elif aType in ('PolyLineShape',):
                self.aEnv['UNOTYPE'] = "PolyLineShape"
                self.aEnv['ENTITYPE'] = "LINE"
                self.aEnv['STEP'] = 1
                self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("VERTEX",)
                for aPolygon in aShape.PolyPolygon:
                    aCoords.append([])
                    aCoords[-1].append(self.compute("CoordFromAbsOrd",\
                                                    aShape.PolyPolygon[0][0],\
                                                    aShape.PolyPolygon[0][1],\
                                                    0,\
                                                    nSign * self.aEnv['OFFSET']))
                    for i in range(len(aPolygon)-2):
                        aCoords[-1].append(uno.createUnoStruct("com.sun.star.awt.Point"))
                        aCoords[-1][-1].X, aCoords[-1][-1].Y = self.compute("InterLines2",\
                                                                aPolygon[i],\
                                                                aPolygon[i+1],\
                                                                aPolygon[i+1],\
                                                                aPolygon[i+2],\
                                                                nSign * self.aEnv['OFFSET'],\
                                                                nSign * self.aEnv['OFFSET'])[2:]
                    aDist = self.compute("toPolarCoord", aShape.PolyPolygon[0][-2], aShape.PolyPolygon[0][-1])[0]
                    aCoords[-1].append(self.compute("CoordFromAbsOrd",\
                                                    aShape.PolyPolygon[0][-2],\
                                                    aShape.PolyPolygon[0][-1],\
                                                    aDist,\
                                                    nSign * self.aEnv['OFFSET']))
            # ................................
            elif aType in ('PolyPolygonShape',):
                self.aEnv['UNOTYPE'] = "PolyPolygonShape"
                self.aEnv['ENTITYPE'] = "LINE"
                self.aEnv['STEP'] = 1
                self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("VERTEX",)
                for aPolygon in aShape.PolyPolygon:
                    aCoords.append([])
                    aCoords[-1].append(uno.createUnoStruct("com.sun.star.awt.Point"))
                    aCoords[-1][-1].X, aCoords[-1][-1].Y = self.compute("InterLines2",\
                                                                        aPolygon[-2],\
                                                                        aPolygon[-1],\
                                                                        aPolygon[0],\
                                                                        aPolygon[1],\
                                                                        nSign * self.aEnv['OFFSET'],\
                                                                        nSign * self.aEnv['OFFSET'])[2:]
                    for i in range(len(aPolygon)-2):
                        aCoords[-1].append(uno.createUnoStruct("com.sun.star.awt.Point"))
                        aCoords[-1][-1].X, aCoords[-1][-1].Y = self.compute("InterLines2",\
                                                                            aPolygon[i],\
                                                                            aPolygon[i+1],\
                                                                            aPolygon[i+1],\
                                                                            aPolygon[i+2],\
                                                                            nSign * self.aEnv['OFFSET'],\
                                                                            nSign * self.aEnv['OFFSET'])[2:]
                    aCoords[-1].append(uno.createUnoStruct("com.sun.star.awt.Point"))
                    aCoords[-1][-1].X, aCoords[-1][-1].Y = aCoords[-1][0].X, aCoords[-1][0].Y
            # ........................................
            elif aType in ('EllipseShape',):
                aCoords.append([])
                self.aEnv['CIRCLEKIND'] = aShape.CircleKind
                self.aEnv['UNOTYPE'] = "EllipseShape"
                aCoords[0].extend([uno.createUnoStruct("com.sun.star.awt.Point") for i in range(2)])
                aCoords[0][0].X = self.aEnv['SPOTTEDSHAPE'][2].X
                aCoords[0][0].Y = self.aEnv['SPOTTEDSHAPE'][2].Y
                self.aEnv['SPOTTEDSHAPE'][3].Width = self.aEnv['SPOTTEDSHAPE'][3].Width + nSign * self.aEnv['OFFSET']
                self.aEnv['SPOTTEDSHAPE'][3].Height = self.aEnv['SPOTTEDSHAPE'][3].Height + nSign * self.aEnv['OFFSET']
                # .........................................................................................................
                if self.aEnv['SPOTTEDSHAPE'][1] in ("CircleShape", "EllipseShape", "ArcCircleShape", "ArcEllipseShape"):
                    self.aEnv['ENTITYPE'] = "CIRCLE"
                    self.aEnv['STEP'] = 1
                    self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("RADIUS",)
                    aPoint = self.compute("PointOnEllipse", None, float(0), *self.aEnv['SPOTTEDSHAPE'][2:5])[0]
                    aCoords[0][1].X = aPoint.X
                    aCoords[0][1].Y = aPoint.Y
                # .........................................................
                if self.aEnv['SPOTTEDSHAPE'][1] in ("EllipseShape", "ArcCircleShape", "ArcEllipseShape"):
                    aCoords[0].extend([uno.createUnoStruct("com.sun.star.awt.Point") for i in range(1)])
                    self.aEnv['ENTITYPE'] = "ELLIPSE"
                    self.aEnv['STEP'] = 2
                    self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("RADIUS", "RADIUS2")
                    aPoint = self.compute("PointOnEllipse", None, math.pi/2, *self.aEnv['SPOTTEDSHAPE'][2:5])[0]
                    aCoords[0][2].X = aPoint.X
                    aCoords[0][2].Y = aPoint.Y
                # ..........................................................................
                if self.aEnv['SPOTTEDSHAPE'][1] in ("ArcCircleShape", "ArcEllipseShape"):
                    aCoords[0].extend([uno.createUnoStruct("com.sun.star.awt.Point") for i in range(2)])
                    self.aEnv['ENTITYPE'] = "ELLARC"
                    self.aEnv['STEP'] = 4
                    self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("RADIUS", "RADIUS2", "STARTANGLE", "ENDANGLE")
                    aPoint = self.compute("PointOnEllipse", None, self.aEnv['SPOTTEDSHAPE'][5], *self.aEnv['SPOTTEDSHAPE'][2:5])[0]
                    aCoords[0][3].X = aPoint.X
                    aCoords[0][3].Y = aPoint.Y
                    aPoint = self.compute("PointOnEllipse", None, self.aEnv['SPOTTEDSHAPE'][6], *self.aEnv['SPOTTEDSHAPE'][2:5])[0]
                    aCoords[0][4].X = aPoint.X
                    aCoords[0][4].Y = aPoint.Y
                # .......................................................
                if self.aEnv['SPOTTEDSHAPE'][1] in ("ArcCircleShape",):
                    del aCoords[0][2]
                    self.aEnv['ENTITYPE'] = "ARC"
                    self.aEnv['STEP'] = 3
                    self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("RADIUS", "STARTANGLE", "ENDANGLE")
            # ..............................
            elif aType in ('RectangleShape',):
                aCoords.append([])
                self.aEnv['UNOTYPE'] = "RectangleShape"
                aCoords[0].extend([uno.createUnoStruct("com.sun.star.awt.Point") for i in range(3)])
                self.aEnv['ENTITYPE'] = "RECTANGLE"
                self.aEnv['STEP'] = 2
                self.aEnv['MODE'] = (self.aEnv['MODE'][0],) + ("WIDTH", "HEIGHT")
                aSize = aShape.getSize()
                aSize.Width = aSize.Width - 2 * nSign * self.aEnv['OFFSET']
                aSize.Height = aSize.Height - 2 * nSign * self.aEnv['OFFSET']
                if aSize.Width < 0 or aSize.Height < 0:
                    self.aEnv['STEP'] = 0
                else:
                    aULCCoord = aShape.getPosition()
                    fAngle1 = aShape.RotateAngle * math.pi / 18000
                    aULCCoord.X = aULCCoord.X + nSign * self.aEnv['OFFSET'] * (math.cos(fAngle1) + math.cos(fAngle1 - math.pi / 2))
                    aULCCoord.Y = aULCCoord.Y - nSign * self.aEnv['OFFSET'] * (math.sin(fAngle1) + math.sin(fAngle1 - math.pi / 2))
                    fCosW = aSize.Width * math.cos(fAngle1) / 2
                    fSinW = -aSize.Width * math.sin(fAngle1) / 2
                    fCosH = aSize.Height * math.cos(fAngle1 - math.pi / 2) / 2
                    fSinH = -aSize.Height * math.sin(fAngle1 - math.pi / 2) / 2
                    aCoords[0][0].X, aCoords[0][0].Y = aULCCoord.X + fCosW + fCosH, aULCCoord.Y + fSinW + fSinH
                    aCoords[0][1].X, aCoords[0][1].Y = aULCCoord.X + 2 * fCosW + fCosH, aULCCoord.Y + 2 * fSinW + fSinH
                    aCoords[0][2].X, aCoords[0][2].Y = aULCCoord.X + fCosW, aULCCoord.Y + fSinW
                    if aShape.CornerRadius:
                        self.aEnv['CORNERRADIUS'] = aShape.CornerRadius - nSign * self.aEnv['OFFSET']
                        if self.aEnv['CORNERRADIUS'] < 0:
                            self.aEnv['CORNERRADIUS'] = 0
                    else:
                        self.aEnv['CORNERRADIUS'] = 0
        else:
            aType = "----"
            self.aEnv['STEP'] = 0
            self.removeTempShape()
            self.aEnv['SPOTTEDID'] = None
            aCoords = [[]]
        sText = "%s ==> %s" % (self.aEnv['COMMENT'][0], aType)
        self.aEnv['STATUSBARCONTROL'][-1].setText(sText)
        self.aEnv['COORDS'] = tuple([tuple(c) for c in aCoords])
        aNamedValues[0].Value = tuple(self.aEnv.items())
        return aNamedValues
    # ----------------------
    # remove temporary shape
    # ----------------------
    def removeTempShape(self):
        if self.aEnv['SHAPE']:
            aGroupShape = self.aEnv['CONTROLLER'].getCurrentPage()
            for i in self.aEnv['SPOTTEDID'][:-2]:
                aGroupShape = aGroupShape.getByIndex(i)
            aGroupShape.remove(self.aEnv['SHAPE'])
            self.aEnv['SHAPE'] = None
        return
# pythonloader looks for a static g_ImplementationHelper variable
g_ImplementationHelper = unohelper.ImplementationHelper()

g_ImplementationHelper.addImplementation(OffsetDraw,                                 # UNO object class
                                         "org.openoffice.comp.pyuno.deneux.OffsetDraw", # implemenation name
                                         ("org.openoffice.comp.pyuno.OffsetDraw",),)    # list of implemented services
